from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .AdaptiveMeshConstraintState import AdaptiveMeshConstraintState
from ..UtilityAndView.abaqusConstants import SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class VelocityAdaptiveMeshConstraintState(AdaptiveMeshConstraintState):
    """The VelocityAdaptiveMeshConstraintState object stores the propagating data for an
    Arbitrary Lagrangian Eularian (ALE) style velocity adaptive mesh constraint in a step.
    One instance of this object is created internally by the VelocityAdaptiveMeshConstraint
    object for each step. The instance is also deleted internally by the
    VelocityAdaptiveMeshConstraint object.
    The VelocityAdaptiveMeshConstraintState object has no constructor or methods.
    The VelocityAdaptiveMeshConstraintState object is derived from the
    AdaptiveMeshConstraintState object.

    .. note::
        This object can be accessed by::

            import step
            mdb.models[name].steps[name].adaptiveMeshConstraintStates[name]

        The corresponding analysis keywords are:

        - ADAPTIVE MESH CONSTRAINT
    """

    #: A Float specifying the velocity component in the 1-direction.
    v1: Optional[float] = None

    #: A Float specifying the velocity component in the 2-direction.
    v2: Optional[float] = None

    #: A Float specifying the velocity component in the 3-direction.
    v3: Optional[float] = None

    #: A Float specifying the rotational velocity component about the 1-direction.
    vr1: Optional[float] = None

    #: A Float specifying the rotational velocity component about the 2-direction.
    vr2: Optional[float] = None

    #: A Float specifying the rotational velocity component about the 3-direction.
    vr3: Optional[float] = None

    #: A SymbolicConstant specifying the propagation state of the velocity component in the
    #: 1-direction. Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    v1State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the velocity component in the
    #: 2-direction. Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    v2State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the velocity component in the
    #: 3-direction. Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    v3State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the rotational velocity component
    #: about the 1-direction. Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    vr1State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the rotational velocity component
    #: about the 2-direction. Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    vr2State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the rotational velocity component
    #: about the 3-direction. Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    vr3State: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the amplitude reference. Possible
    #: values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    amplitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the AdaptiveMeshConstraintState
    #: object. Possible values
    #: are: NOT_YET_ACTIVE, CREATED, PROPAGATED, MODIFIED, DEACTIVATED, NO_LONGER_ACTIVE, TYPE_NOT_APPLICABLE
    #: INSTANCE_NOT_APPLICABLE, PROPAGATED_FROM_BASE_STATE, MODIFIED_FROM_BASE_STATE, DEACTIVATED_FROM_BASE_STATE,
    #: BUILT_INTO_MODES
    status: Optional[SymbolicConstant] = None

    #: A String specifying the name of the amplitude reference. The String is empty if the
    #: adaptive mesh constraint has no amplitude reference.
    amplitude: str = ""

    @abaqus_method_doc
    def __init__(
        self,
        v1: Optional[float] = None,
        v2: Optional[float] = None,
        v3: Optional[float] = None,
        vr1: Optional[float] = None,
        vr2: Optional[float] = None,
        vr3: Optional[float] = None,
        v1State: Optional[Literal[C.UNSET, C.SET, C.FREED, C.UNCHANGED, C.MODIFIED]] = None,
        v2State: Optional[Literal[C.UNSET, C.SET, C.FREED, C.UNCHANGED, C.MODIFIED]] = None,
        v3State: Optional[Literal[C.UNSET, C.SET, C.FREED, C.UNCHANGED, C.MODIFIED]] = None,
        vr1State: Optional[Literal[C.UNSET, C.SET, C.FREED, C.UNCHANGED, C.MODIFIED]] = None,
        vr2State: Optional[Literal[C.UNSET, C.SET, C.FREED, C.UNCHANGED, C.MODIFIED]] = None,
        vr3State: Optional[Literal[C.UNSET, C.SET, C.FREED, C.UNCHANGED, C.MODIFIED]] = None,
        amplitudeState: Optional[Literal[C.UNSET, C.SET, C.FREED, C.UNCHANGED, C.MODIFIED]] = None,
        status: Optional[
            Literal[
                C.NOT_YET_ACTIVE,
                C.PROPAGATED_FROM_BASE_STATE,
                C.DEACTIVATED_FROM_BASE_STATE,
                C.DEACTIVATED,
                C.MODIFIED_FROM_BASE_STATE,
                C.PROPAGATED,
                C.NO_LONGER_ACTIVE,
                C.CREATED,
                C.INSTANCE_NOT_APPLICABLE,
                C.BUILT_INTO_MODES,
                C.TYPE_NOT_APPLICABLE,
                C.MODIFIED,
            ]
        ] = None,
        amplitude: str = "",
    ):
        """The VelocityAdaptiveMeshConstraintState object stores the propagating data for an
        Arbitrary Lagrangian Eularian (ALE) style velocity adaptive mesh constraint in a step.
        One instance of this object is created internally by the VelocityAdaptiveMeshConstraint
        object for each step. The instance is also deleted internally by the
        VelocityAdaptiveMeshConstraint object.
        The VelocityAdaptiveMeshConstraintState object has no constructor or methods.
        The VelocityAdaptiveMeshConstraintState object is derived from the
        AdaptiveMeshConstraintState object.

        .. note::
            This function can be accessed by::

                mdb.models[name].steps[name].VelocityAdaptiveMeshConstraintState

        Parameters
        ----------
        v1
            A Float specifying the velocity component in the 1-direction.
        v2
            A Float specifying the velocity component in the 2-direction.
        v3
            A Float specifying the velocity component in the 3-direction.
        vr1
            A Float specifying the rotational velocity component about the 1-direction.
        vr2
            A Float specifying the rotational velocity component about the 2-direction.
        vr3
            A Float specifying the rotational velocity component about the 3-direction.
        v1State
            A SymbolicConstant specifying the propagation state of the velocity component in the  1-direction.
            Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
        v2State
            A SymbolicConstant specifying the propagation state of the velocity component in the  2-direction.
            Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
        v3State
            A SymbolicConstant specifying the propagation state of the velocity component in the  3-direction.
            Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
        vr1State
            A SymbolicConstant specifying the propagation state of the rotational velocity component  about the
            1-direction. Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
        vr2State
            A SymbolicConstant specifying the propagation state of the rotational velocity component  about the
            2-direction. Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
        vr3State
            A SymbolicConstant specifying the propagation state of the rotational velocity component  about the
            3-direction. Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
        amplitudeState
            A SymbolicConstant specifying the propagation state of the amplitude reference. Possible  values are UNSET,
            SET, UNCHANGED, FREED, and MODIFIED.
        status
            A SymbolicConstant specifying the propagation state of the AdaptiveMeshConstraintState  object. Possible
            values  are:
            - NOT_YET_ACTIVE
            - CREATED
            - PROPAGATED
            - MODIFIED
            - DEACTIVATED
            - NO_LONGER_ACTIVE
            - TYPE_NOT_APPLICABLE
            - INSTANCE_NOT_APPLICABLE
            - PROPAGATED_FROM_BASE_STATE
            - MODIFIED_FROM_BASE_STATE
            - DEACTIVATED_FROM_BASE_STATE
            - BUILT_INTO_MODES
        amplitude
            A String specifying the name of the amplitude reference. The String is empty if the  adaptive mesh
            constraint has no amplitude reference.
        """
        super().__init__(amplitudeState, status, amplitude)



import re
from typing import Tuple

from . import __version__ as version

version = version[:4]


class AbaqusDoc:
    @classmethod
    def _class_or_module_link(
        cls,
        type: str,
        class_or_module_name: str,
        prefix="",
        suffix="",
        label=None,
    ) -> Tuple[str, str]:
        """Generate a link to the Abaqus class documentation.

        Parameters
        ----------
        type : 'class' or 'module'
            Type of the object, 'class' or 'module'.
        class_or_module_name : str
            The name of the class or module.
        prefix : str
            The prefix to the class or module name.
        suffix : str
            The suffix to the class name.
        label : str
            The label to use for the link.

        Returns
        -------
        str
            The link to the documentation.
        str
            The link with label.
        """
        if label is None:
            label = class_or_module_name
        module_name = f"{type if type == 'module' else ''}{class_or_module_name.lower()}"
        link = (
            f"https://help.3ds.com/{version}/English/DSSIMULIA_Established/SIMACAEKERRefMap/"
            f"simaker-c-{prefix}{module_name}{suffix}pyc.htm?contextscope=all"
        )
        return link, f"Check `{label} on help.3ds.com/{version} <{link}>`__."

    @classmethod
    def _method_or_function_link(
        cls,
        type: str,
        class_or_module_name: str,
        method_or_function_name: str,
        prefix="",
        suffix="",
        label=None,
    ) -> Tuple[str, str]:
        """Generate a link to the Abaqus function documentation.

        Parameters
        ----------
        type : 'method' or 'function'
            Type of the object, 'method' or 'function'.
        class_or_module_name : str
            The name of the module.
        method_or_function_name : str
            The name of the function.
        prefix : str
            The prefix to the class or module name.
        suffix : str
            The suffix to the class name.
        label : str
            The label to use for the link.

        Returns
        -------
        str
            The link to the documentation.
        str
            The link with label.
        """
        # For methods that start with a capital letter, the link is to the corresponding class documentation.
        if method_or_function_name[0].isupper():
            return cls._class_or_module_link("class", method_or_function_name, prefix, suffix, method_or_function_name)
        class_name = f"{type if type == 'function' else ''}{class_or_module_name.lower()}"
        function_prefix = prefix if type == "function" else ""
        if type == "class" and method_or_function_name == "__init__":
            method_or_function_name = class_or_module_name
        link = (
            f"https://help.3ds.com/{version}/English/DSSIMULIA_Established/SIMACAEKERRefMap/"
            f"simaker-c-{prefix}{class_name}{suffix}pyc.htm?contextscope=all"
            f"#simaker-{function_prefix}{class_name}{method_or_function_name.lower()}pyc"
        )
        if method_or_function_name == class_or_module_name and type == "class":
            signature = f"{class_or_module_name}()"
        else:
            signature = f"{class_or_module_name}.{method_or_function_name}()"
        if label is None:
            label = signature
        return link, f"Check `{label} on help.3ds.com/{version} <{link}>`__."

    @classmethod
    def _add_link_in_class_or_module_docstring(
        cls,
        type: str,
        class_or_module_name: str,
        docstring: str,
        prefix="",
        suffix="",
        label=None,
    ) -> str:
        """Add a link to the Abaqus documentation to the docstring for classes or modules.

        Parameters
        ----------
        type : 'class' or 'module'
            Type of the object, 'class' or 'module'.
        class_or_module_name : str
            The name of the class or module.
        docstring : str
            The docstring.
        prefix : str
            The prefix to the class or module name.
        suffix : str
            The suffix to the class name.
        label : str
            The label to use for the link.

        Returns
        -------
        str
            The docstring with the link to the Abaqus documentation.
        """
        if not docstring:
            return docstring
        link, link_with_label = cls._class_or_module_link(type, class_or_module_name, prefix, suffix, label)
        return (
            docstring.rstrip()
            + "\n\n"
            + " " * (0 if type == "module" else 4)
            + ".. note::\n"
            + " " * (4 if type == "module" else 8)
            + link_with_label
        )

    @classmethod
    def _add_link_in_method_or_function_docstring(
        cls,
        type: str,
        class_or_module_name: str,
        method_or_function_name: str,
        docstring: str,
        prefix="",
        suffix="",
        label=None,
    ) -> str:
        """Add a link to the Abaqus documentation to the docstring for methods or functions.

        Parameters
        ----------
        type : 'method' or 'function'
            Type of the object, 'method' or 'function'.
        class_or_module_name : str
            The name of the module.
        method_or_function_name : str
            The name of the function.
        docstring : str
            The docstring.
        prefix : str
            The prefix to the class or module name.
        suffix : str
            The suffix to the class name.
        label : str
            The label to use for the link.

        Returns
        -------
        str
            The docstring with the link to the Abaqus documentation.
        """
        if not docstring:
            return docstring
        link, link_with_label = cls._method_or_function_link(
            type, class_or_module_name, method_or_function_name, prefix, suffix, label
        )
        return re.sub(
            r"(\n\s+?)(Parameters\n\s+----------)",
            r"\1" + ".. note::\n" + " " * (8 if type == "function" else 12) + link_with_label + r"\1\2",
            docstring,
        )

    @classmethod
    def add_link_in_class_docstring(
        cls,
        class_name: str,
        docstring: str,
        prefix="",
        suffix="",
        label=None,
    ) -> str:
        """Add a link to the Abaqus documentation to the docstring for classes.

        Parameters
        ----------
        class_name : str
            The name of the class.
        docstring : str
            The docstring.
        prefix : str
            The prefix to the class or module name.
        suffix : str
            The suffix to the class name.
        label : str
            The label to use for the link.

        Returns
        -------
        str
            The docstring with the link to the Abaqus documentation.
        """
        return cls._add_link_in_class_or_module_docstring("class", class_name, docstring, prefix, suffix, label)

    @classmethod
    def add_link_in_module_docstring(
        cls,
        module_name: str,
        docstring: str,
        prefix="",
        suffix="",
        label=None,
    ) -> str:
        """Add a link to the Abaqus documentation to the docstring for modules.

        Parameters
        ----------
        module_name : str
            The name of the module.
        docstring : str
            The docstring.
        prefix : str
            The prefix to the class or module name.
        suffix : str
            The suffix to the class name.
        label : str
            The label to use for the link.

        Returns
        -------
        str
            The docstring with the link to the Abaqus documentation.
        """
        return cls._add_link_in_class_or_module_docstring("module", module_name, docstring, prefix, suffix, label)

    @classmethod
    def add_link_in_function_docstring(
        cls,
        module_name: str,
        function_name: str,
        docstring: str,
        prefix="",
        suffix="",
        label=None,
    ) -> str:
        """Add a link to the Abaqus documentation to the docstring for functions.

        Parameters
        ----------
        module_name : str
            The name of the module.
        function_name : str
            The name of the function.
        docstring : str
            The docstring.
        prefix : str
            The prefix to the class or module name.
        suffix : str
            The suffix to the class name.
        label : str
            The label to use for the link.

        Returns
        -------
        str
            The docstring with the link to the Abaqus documentation.
        """
        return cls._add_link_in_method_or_function_docstring(
            "function", module_name, function_name, docstring, prefix, suffix, label
        )

    @classmethod
    def add_link_in_method_docstring(
        cls,
        class_name: str,
        method_name: str,
        docstring: str,
        prefix="",
        suffix="",
        label=None,
    ) -> str:
        """Add a link to the Abaqus documentation to the docstring for methods.

        Parameters
        ----------
        class_name : str
            The name of the class.
        method_name : str
            The name of the method.
        docstring : str
            The docstring.
        prefix : str
            The prefix to the class or module name.
        suffix : str
            The suffix to the class name.
        label : str
            The label to use for the link.

        Returns
        -------
        str
            The docstring with the link to the Abaqus documentation.
        """
        return cls._add_link_in_method_or_function_docstring(
            "method", class_name, method_name, docstring, prefix, suffix, label
        )


doc = AbaqusDoc


def abaqus_function_doc(func):
    """Add a link to the Abaqus documentation to the docstring of the function."""
    module_name = func.__module__.split(".")[-1]
    func.__doc__ = doc.add_link_in_function_docstring(
        module_name=module_name,
        function_name=func.__name__,
        docstring=func.__doc__,
        prefix="gpr" if module_name.lower().startswith("cae") else "",
        label=f"{module_name}.{func.__name__}",
    )
    return func


def _process_class_name(class_name: str) -> str:
    """Process the class name."""
    # Handle the base class case.
    if class_name.endswith("Base"):
        class_name = class_name[:-4]
    # Handle the class with a suffix.
    if class_name.endswith("Mdb"):
        class_name = "Mdb"
    if class_name.endswith("Model"):
        class_name = "Model"
    if class_name.endswith("Step"):
        class_name = "Step"
    if class_name.endswith("Assembly"):
        class_name = "Assembly"
    if class_name.endswith("Step"):
        class_name = "Step"
    if class_name.endswith("Odb"):
        class_name = "Odb"
    if class_name.endswith("Part"):
        class_name = "Part"
    return class_name


class_suffix = {
    "AdaptivityModel": "kar",
    "AssemblyModel": "asm",
    "EditMeshAssembly": "edt",
    "EditMeshPart": "edt",
    "InteractionModel": "itn",
    "MeshAssembly": "mgn",
    "MeshPart": "mgn",
    "RegionAssembly": "reg",
    "RegionPart": "reg",
}


def abaqus_method_doc(method):
    """Add a link to the Abaqus documentation to the docstring of the method."""
    class_name = method.__qualname__.split(".")[0]
    method.__doc__ = doc.add_link_in_method_docstring(
        class_name=_process_class_name(class_name),
        method_name=method.__name__,
        docstring=method.__doc__,
        prefix="gpr" if class_name.lower().startswith("cae") else "",
        suffix=class_suffix.get(class_name, ""),
        label=f"{class_name}.{method.__name__}",
    )
    return method


def abaqus_class_doc(cls):
    """Add a link to the Abaqus documentation to the docstring of the class."""
    class_name = cls.__name__
    processed_class_name = _process_class_name(class_name)
    cls.__doc__ = doc.add_link_in_class_docstring(
        class_name=processed_class_name,
        docstring=cls.__doc__,
        prefix="gpr" if class_name.lower().startswith("cae") else "",
        suffix=class_suffix.get(class_name, ""),
        label=class_name,
    )
    return cls


@abaqus_function_doc
def foo(a, b):
    """This is a test function.

    Parameters
    ----------
    a : int
        The first parameter.
    b : int
        The second parameter.
    """
    return a + b


@abaqus_class_doc
class Bar:
    """This is a test class."""

    @abaqus_method_doc
    def foo(self, a, b):
        """This is a test method.

        Parameters
        ----------
        a : int
            The first parameter.
        b : int
            The second parameter.
        """
        pass


def test_foo_bar():
    """
    Test the foo function.
    """
    # foo
    print("\n")
    print(foo.__name__)
    print(foo.__doc__)

    # Bar
    print(Bar.__name__)
    print(Bar.__doc__)

    # Bar.foo
    print(Bar.foo.__name__)
    print(Bar.foo.__doc__)



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .PredefinedField import PredefinedField
from ..Region.Region import Region


@abaqus_class_doc
class FluidCavityPressure(PredefinedField):
    """The FluidCavityPressure object stores the data for initial fluid cavity pressures. The
    base class *region* argument can not be specifed with this object.
    The FluidCavityPressure object is derived from the PredefinedField object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].predefinedFields[name]

        The corresponding analysis keywords are:

        - INITIAL CONDITIONS
    """

    #: A Region object on which the **fluidCavity** interaction is specified.
    region: Region = Region()

    #: A String specifying the repository key.
    name: str

    #: A String specifying the name of a Fluid Cavity Interaction.
    fluidCavity: str

    #: A Float specifying the initial fluid pressure.
    fluidPressure: float

    @abaqus_method_doc
    def __init__(self, name: str, fluidCavity: str, fluidPressure: float):
        """This method creates a FluidCavityPressure object.

        .. note::
            This function can be accessed by::

                mdb.models[name].FluidCavityPressure

        Parameters
        ----------
        name
            A String specifying the repository key.
        fluidCavity
            A String specifying the name of a Fluid Cavity Interaction.
        fluidPressure
            A Float specifying the initial fluid pressure.

        Returns
        -------
        FluidCavityPressure
            A FluidCavityPressure object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the FluidCavityPressure object."""
        ...



from typing_extensions import Literal

from .Imperfection import Imperfection
from ..UtilityAndView.abaqusConstants import Boolean, OFF, R
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


class InputImperfection(Imperfection):
    """The InputImperfection object defines geometric imperfection on an assembly region.
    The InputImperfection object is derived from the Imperfection object.

    .. note::
        This object can be accessed by::

        import assembly
        mdb.models[name].rootAssembly.engineeringFeatures.imperfections[name]

        The corresponding analysis keywords are:

        - IMPERFECTION

    .. versionadded:: 2023
        The `InputImperfection` class was added.
    """

    #: A String specifying the repository key.
    name: str

    #: A String specifying the name of the alternate input file containing the imperfection data.
    file: str

    #: A SymbolicConstant specifying the coordinate system. The imperfection values in the alternate input file
    #: being used as input to this option would be treated as perturbation values of respective coordinates.
    #: Possible values are R, C and S. The default value is R.
    system: Literal[C.R, C.C, C.S] = R

    #: A Boolean specifying whether the imperfection is suppressed or not. The default value is OFF.
    suppressed: Boolean = OFF

    def __init__(
        self,
        name: str,
        file: str,
        system: Literal[C.R, C.C, C.S] = R,
    ):
        """This method creates a InputImperfection object.

        .. note::
            This function can be accessed by::

                mdb.models[name].rootAssembly.engineeringFeatures.InputImperfection

        Parameters
        ----------
        name
            A String specifying the repository key.
        file
            A String specifying the name of the alternate input file containing the imperfection data.
        system
            A SymbolicConstant specifying the coordinate system. The imperfection values in the alternate input file
            being used as input to this option would be treated as perturbation values of respective coordinates.
            Possible values are R, C and S. The default value is R.

        Returns
        -------
        InputImperfection
            A InputImperfection object.
        """
        super().__init__(name)

    def setValues(
        self,
        file: str,
        system: Literal[C.R, C.C, C.S] = R,
    ):
        """This method modifies the InputImperfection object.

        Parameters
        ----------
        file
            A String specifying the name of the alternate input file containing the imperfection data.
        system
            A SymbolicConstant specifying the coordinate system. The imperfection values in the alternate input file
            being used as input to this option would be treated as perturbation values of respective coordinates.
            Possible values are R, C and S. The default value is R.
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from ....TestData.BiaxialTestData import BiaxialTestData
from ....TestData.PlanarTestData import PlanarTestData
from ....TestData.SimpleShearTestData import SimpleShearTestData
from ....TestData.UniaxialTestData import UniaxialTestData
from ....TestData.VolumetricTestData import VolumetricTestData
from .....UtilityAndView.abaqusConstants import Boolean, LONG_TERM, OFF
from .....UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class Hyperfoam:
    r"""The Hyperfoam object specifies elastic properties for a hyperelastic foam.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].hyperfoam
            import odbMaterial
            session.odbs[name].materials[name].hyperfoam

        The table data for this object are:
        The items in the table data specify the following for values of nn:

        - :math:`\mu_i` and :math:`\alpha_i` for :math:`i` from 1 to :math:`n`.
        - :math:\nu_i`.
        - Temperature, if the data depend on temperature. Temperature dependence is not allowed for :math:`4\le n\le 6` in an
          Abaqus/Explicit analysis.

        The corresponding analysis keywords are:

        - HYPERFOAM
    """

    #: A BiaxialTestData object.
    biaxialTestData: BiaxialTestData = BiaxialTestData(((),))

    #: A VolumetricTestData object.
    volumetricTestData: VolumetricTestData = VolumetricTestData(((),))

    #: A PlanarTestData object.
    planarTestData: PlanarTestData = PlanarTestData(((),))

    #: A SimpleShearTestData object.
    simpleShearTestData: SimpleShearTestData = SimpleShearTestData(((),))

    #: A UniaxialTestData object.
    uniaxialTestData: UniaxialTestData = UniaxialTestData(((),))

    @abaqus_method_doc
    def __init__(
        self,
        testData: Boolean = OFF,
        poisson: Optional[float] = None,
        n: int = 1,
        temperatureDependency: Boolean = OFF,
        moduli: Literal[C.INSTANTANEOUS, C.LONG_TERM] = LONG_TERM,
        table: tuple = (),
    ):
        """This method creates a Hyperfoam object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].Hyperfoam
                session.odbs[name].materials[name].Hyperfoam

        Parameters
        ----------
        testData
            A Boolean specifying whether test data are supplied. The default value is OFF.
        poisson
            None or a Float specifying the effective Poisson's ratio, νν, of the material. This
            argument is valid only when **testData** = ON. The default value is None.
        n
            An Int specifying the order of the strain energy potential. Possible values are 1 ≤n≤n≤
            6. The default value is 1.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        moduli
            A SymbolicConstant specifying the time-dependence of the material constants. Possible
            values are INSTANTANEOUS and LONG_TERM. The default value is LONG_TERM.
        table
            A sequence of sequences of Floats specifying the items described below. This argument is
            valid only when **testData** = OFF. The default value is an empty sequence.

        Returns
        -------
        Hyperfoam
            A Hyperfoam object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the Hyperfoam object.

        Raises
        ------
        RangeError
        """
        ...
